/*
 *  Amiga Linux/m68k Loader
 *
 *  Fake a boot from a specific device under AmigaOS (for debugging)
 *
 *   Copyright 1996 by Geert Uytterhoeven
 *		       (Geert.Uytterhoeven@cs.kuleuven.ac.be)
 *
 *  --------------------------------------------------------------------------
 *
 *  Usage (under AmigaOS):
 *
 *	fakeboot <device> <unit> [<offset>]
 *
 *  With:
 *
 *	<device>    : device name
 *	<unit>	    : unit number
 *	<offset>    : boot block offset (default: 0)
 *
 *  --------------------------------------------------------------------------
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License.  See the file COPYING in the main directory of the Linux
 *  distribution for more details.
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>
#include <devices/trackdisk.h>
#include <proto/exec.h>


struct MsgPort *Port = NULL;
struct IOStdReq *Request = NULL;


u_long BootBlock[256];


static __inline u_long CallBootCode(u_long BootBlock[], struct IOStdReq *Request)
{
    register u_long _res __asm("d0");
    register u_long *a0 __asm("a0") = BootBlock;
    register struct IOStdReq *a1 __asm("a1") = Request;
    register struct ExecBase *a6 __asm("a6") = SysBase;

    __asm __volatile ("jsr a0@"
		      : "=r" (_res)
		      : "r" (a0), "r" (a1), "r" (a6)
		      : "a0", "a1", "d0", "d1", "memory");
    return(_res);
}


void __attribute__ ((noreturn)) Die(const char *fmt, ...)
{
    va_list ap;

    fflush(stdout);
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);

    if (Request) {
	CloseDevice((struct IORequest *)Request);
	DeleteIORequest(Request);
    }
    if (Port)
	DeleteMsgPort(Port);

    exit(1);
}


int main(int argc, char *argv[])
{
    char *device;
    u_long unit, offset, res;
    int error;

    if (argc != 3 && argc != 4) {
	printf("Usage: %s <device> <unit> [<offset>]\n", argv[0]);
	return(10);
    }

    device = argv[1];
    unit = strtoul(argv[2], NULL, 0);
    offset = argc == 4 ? strtoul(argv[3], NULL, 0) : 0;

    if (!(Port = CreateMsgPort()))
	Die("Couldn't create MsgPort\n");
    if (!(Request = CreateIORequest(Port, sizeof(struct IOStdReq))))
	Die("Couldn't create IORequest\n");
    if (OpenDevice(device, unit, (struct IORequest *)Request, 0))
	Die("Couldn't open Device\n");

    Request->io_Command = CMD_READ;
    Request->io_Flags = IOF_QUICK;
    Request->io_Length = 1024;
    Request->io_Data = BootBlock;
    Request->io_Offset = offset;
    if ((error = DoIO((struct IORequest *)Request)))
	Die("Error reading Bootcode: %d\n", error);

    Request->io_Command = TD_MOTOR;
    Request->io_Flags = IOF_QUICK;
    Request->io_Length = 0;
    DoIO((struct IORequest *)Request);

    res = CallBootCode(BootBlock, Request);
    Die("Result = 0x%08lx\n", res);
}
