/*
 *  Amiga Linux/m68k Loader
 *
 *   Copyright 1995 by Geert Uytterhoeven
 *
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License.  See the file COPYING in the main directory of the Linux
 *  distribution for more details.
 */


#include <asm/setup.h>

#include "linuxboot.h"


#define VERSION			"Amiga Linux/m68k Loader Version 1.0 " \
				"[Amiboot " AMIBOOT_VERSION "] (9/9/96)\n" \
				"(C) Copyright 1995 by Geert Uytterhoeven\n"


#define LILO_ID			(0x4C494C4F)	/* `LILO' */
#define LILO_MAPVECTORID	(0x4D415056)	/* `MAPV' */

#define LILO_BLOCKSIZE		(1024)

#define TRUE			(1)
#define FALSE			(0)


#define LILO_CONFIGFILE		"/etc/lilo.conf"
#define LILO_MAPFILE		"/boot/map"
#define LILO_LOADERTEMPLATE	"/boot/loader"
#define LILO_LOADERFILE		"/boot/loader.patched"
#define LILO_BACKUPFILEPREFIX	"/boot/backup"


#define ID_BOOT_DISK		(0x424F4F55)	/* `BOOU' */
#define ID_OFS_DISK		(0x444F5300)	/* `DOS\0' */
#define ID_FFS_DISK		(0x444F5301)	/* `DOS\1' */
#define ID_OFS_INTL_DISK	(0x444F5302)	/* `DOS\2' */
#define ID_FFS_INTL_DISK	(0x444F5303)	/* `DOS\3' */
#define ID_OFS_DC_DISK		(0x444F5304)	/* `DOS\4' */
#define ID_FFS_DC_DISK		(0x444F5305)	/* `DOS\5' */
#define ID_MUFS_DISK		(0x6d754653)	/* `muFS' */
#define ID_MU_OFS_DISK		(0x6d754600)	/* `muF\0' */
#define ID_MU_FFS_DISK		(0x6d754601)	/* `muF\1' */
#define ID_MU_OFS_INTL_DISK	(0x6d754602)	/* `muF\2' */
#define ID_MU_FFS_INTL_DISK	(0x6d754603)	/* `muF\3' */
#define ID_MU_OFS_DC_DISK	(0x6d754604)	/* `muF\4' */
#define ID_MU_FFS_DC_DISK	(0x6d754605)	/* `muF\5' */


struct BootBlock {
    u_long ID;
    u_long Checksum;
    u_long DosBlock;
    u_long Data[252];
    u_long LiloID;
};


    /*
     *	Boot Operating Systems
     */

#define BOS_AMIGA		(0)	/* AmigaOS */
#define BOS_LINUX		(1)	/* Linux/m68k */
#define BOS_NETBSD		(2)	/* Future Expansion :-) */


    /*
     *	Tag Record
     *
     *	The Map File consists of a concatenation of Tag Records
     *
     *	    Tag				Size	Data
     *	    ---				----	----
     *
     *	File Identification:
     *
     *	    TAG_LILO			0	-
     *
     *	Boot Options:
     *
     *	    TAG_HEADER			0	-
     *	    TAG_DEFAULT				char []
     *	    TAG_AUTO			4	u_long
     *	    TAG_TIME_OUT		4	u_long
     *	    TAG_AUX			4	u_long
     *	    TAG_BAUD			4	u_long
     *	    TAG_MASTER_PASSWORD			char []
     *	    TAG_DEBUG			4	u_long
     *	    TAG_PROMPT				char []
     *	    TAG_KICKRANGE		4	u_short [2]
     *	    TAG_HEADER_END		0	-
     *
     *	Boot Records:
     *
     *	    TAG_BOOT_RECORD			char []
     *	    TAG_ALIAS				char []
     *	    TAG_OS_TYPE			4	u_long
     *	    TAG_KERNEL				char []
     *	    TAG_ARGUMENTS			char []
     *	    TAG_PASSWORD			char []
     *	    TAG_CHIP_RAM_SIZE		4	u_long
     *	    TAG_FAST_RAM_CHUNK		8	u_long [2]
     *	    TAG_MODEL			4	u_long
     *	    TAG_RAMDISK				char []
     *	    TAG_BOOT_RECORD_END		0	-
     *
     *	File Records:
     *
     *	    TAG_FILE_DEF			char []
     *	    TAG_VECTOR				u_long []
     *	    TAG_FILE_DEF_END		0	-
     *
     *	End of File:
     *
     *	    TAG_EOF			0	-
     *
     */

struct TagRecord {
    u_long Tag;			/* Tag Identifier */
    u_long Size;		/* Must be a multiple of 4 bytes */
    u_long Data[0];		/* Size bytes of Tag Data */
};


    /*
     *	Tag Identifiers
     */

#define TAG_LILO		LILO_ID	/* File Identification */
#define TAG_EOF			(0)	/* End of File */

#define TAG_HEADER		(1000)	/* Start of Header */
#define TAG_HEADER_END		(1001)	/* End of Header */
#define TAG_DEFAULT		(1002)	/* Default Boot Image */
#define TAG_AUTO		(1003)	/* Automatic Boot Flag */
#define TAG_TIME_OUT		(1004)	/* Time Out Value */
#define TAG_AUX			(1005)	/* Auxiliary Console */
#define TAG_BAUD		(1006)	/* Baud for Auxiliary Console */
#define TAG_MASTER_PASSWORD	(1007)	/* Master Password */
#define TAG_DEBUG		(1008)	/* Debug Flag */
#define TAG_PROMPT		(1009)	/* Boot Prompt */
#define TAG_KICKRANGE		(1010)	/* Valid Kickstart Version Range */

#define TAG_BOOT_RECORD		(2000)	/* Start of Boot Record */
#define TAG_BOOT_RECORD_END	(2001)	/* End of Boot Record */
#define TAG_ALIAS		(2002)	/* Alias Label Name */
#define TAG_OS_TYPE		(2003)	/* Operating System Type */
#define TAG_KERNEL		(2004)	/* Kernel Image */
#define TAG_ARGUMENTS		(2005)	/* Boot Arguments */
#define TAG_PASSWORD		(2006)	/* Password */
#define TAG_CHIP_RAM_SIZE	(2007)	/* Chip RAM Size */
#define TAG_FAST_RAM_CHUNK	(2008)	/* Fast RAM Chunk */
#define TAG_MODEL		(2009)	/* Amiga Model */
#define TAG_RAMDISK		(2010)	/* Ramdisk Image */

#define TAG_FILE_DEF		(3000)	/* Start of File Definition */
#define TAG_FILE_DEF_END	(3001)	/* End of File Definition */
#define TAG_VECTOR		(3002)	/* File Vector */


    /*
     *	Boot Options
     */

struct BootOptions {
    const char *Default;		/* Default Boot Image */
    const u_long *Auto;			/* Automatic Boot */
    const u_long *TimeOut;		/* In Seconds, Zero is Infinite */
    const u_long *Aux;			/* Use Auxiliary Console too */
    const u_long *Baud;			/* Baud for Auxiliary Console */
    const char *MasterPassword;		/* Master Password */
    const u_long *Debug;		/* Debug Flag */
    const char *Prompt;			/* Boot Prompt */
    const u_short *KickRange;		/* Valid Kickstart Version Range */
};


    /*
     *	Boot Record Definitions
     */

struct BootRecord {
    struct BootRecord *Next;		/* Next Boot Record */
    const char *Label;			/* Boot Record Label */
    const char *Alias;			/* Alias Label Name */
    const u_long *OSType;		/* Operating System Type */
    const char *Kernel;			/* Kernel Image */
    const char *Args;			/* Boot Arguments */
    const char *Password;		/* Password */
    const u_long *ChipSize;		/* Chip RAM Size */
    const u_long *FastChunks[NUM_MEMINFO];	/* Fast RAM Chunks */
    const u_long *Model;		/* Amiga Model */
    const char *Ramdisk;		/* Ramdisk */
};


#define PATH_SIZE	256		/* Should be sufficient */


    /*
     *	File Definition
     */

struct FileDef {
    struct FileDef *Next;		/* Next File Definition */
    const char *Path;			/* File Path */
    const u_long *Vector;		/* File Vector */
};

#define arraysize(x)	(sizeof(x)/sizeof(*(x)))


    /*
     *	Alternate Boot Partition
     *
     *	Warning: this definition is hardcoded in bootcode.s too!
     */

#define MAXALTDEVSIZE	(32)


    /*
     *	Configuration File Parsing
     */

struct Config {
    const char *BootDevice;
    const char *AltDeviceName;
    const u_long *AltDeviceUnit;
    struct BootOptions Options;
    struct BootRecord *Records;
    struct FileDef *Files;
};
